<?php

namespace App\Base\Redis;

use Predis\Client;

class KarmaRepository
{
    protected $redis;
    protected $prefix = 'karma|';

    public function __construct(Client $redis)
    {
        $this->redis = $redis;
    }

    public function get(string $key): ?Karma
    {
        $v = $this->redis->hgetall($key);
        $a = explode('|', $key);

        if ($v === null) return null;

        $kv = new Karma();
        $kv->ip = $a[1];
        $kv->bad = $v['bad'];
        $kv->good = $v['good'];
        $kv->connections = $v['connections'];

        $ttl = $this->redis->ttl($key);
        $dt = new \DateTime();
        $dt->setTimestamp(time() + $ttl);
        $kv->ttl = $dt;

        return $kv;
    }

    /**
     * for testing only
     */
    public function add(string $ip, int $bad, int $good, int $connections)
    {
        $this->redis->hmset($this->prefix.$ip, ['good' => $good, 'bad' => $bad, 'connections' => $connections]);
    }

    public function delete(Karma $kv): void
    {
        $this->redis->del(['karma|'.$kv->ip]);
    }

    public function deleteAll(): void
    {
        $arr = [];
        foreach ($this->redis->keys($this->prefix.'*') as $key) {
            $arr[] = $key;
        }

        if (!empty($arr)) {
            $this->redis->del($arr);
        }
    }

    /**
     * @param string $query
     * @return Karma[]
     */
    public function list(?string $query = null, int $page = 0, int $onPage = 10): array
    {
        $cursor = 0;
        $i = 0;

        /*
         * INVALID algo
        do {
            $result = $this->redis->scan(0, ['match' => $this->prefix . $query, 'count' => $onPage]);

            if ($page == $i) {
                return array_map(function($key) { return $this->get($key); }, $result[1]);
            }
            $i++;
        } while ($cursor != 0);

        return [];
        */

        // TODO rewrite when this became problem
        $all_keys = $this->redis->keys($this->prefix.$query);
        $keys = array_slice($all_keys, $onPage*$page, $onPage);
        return array_map(function($key) { return $this->get($key); }, $keys);
    }

    public function persist(): void
    {
        try {
            $this->redis->bgsave();
        } catch (\Exception $e) {}
    }
}
